% WIP: optimization function and automation for choosing a mu for
% regulation of angles

%*-------------------------------------------------------------------------------------*
%|                                                                                     |
%|  Except where otherwise noted, this work is licensed under a                        |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0                          |
%|  International (CC BY-NC-SA 4.0) license.                                           |
%|                                                                                     |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)                  |
%|                                                                                     |
%|      Author: CXS group, PSI                                                         |
%*------------------------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If this code, or subfunctions or parts of it, is used for research in a
%   publication or if it is fully or partially rewritten for another
%   computing language the authors and institution should be acknowledged
%   in written form and additionally you should cite:
%     M. Liebi, M. Georgiadis, A. Menzel, P. Schneider, J. Kohlbrecher,
%     O. Bunk, and M. Guizar-Sicairos, “Nanostructure surveys of
%     macroscopic specimens by small-angle scattering tensor tomography,”
%     Nature 527, 349-352 (2015).   (doi:10.1038/nature16056)
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its
%    proper use and the correctness of the results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
error('Please run this script by section');

%% Step 2.1 Load the data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
base_path = '~/Data10/';                    % = '~/Data10/' for online analysis,
% provide the path for offline analysis
% Ex: '/das/work/p16/p16649/'
sample_name = 'sample_name';                % name given in the saxs_caller_template
params.add_name = 'add_id';       % additional name the optimizations: = [ ] if not needed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath ../
addpath utilities/
params.sample_name = sample_name;
% create folders to save data
params.optimization_output_path = fullfile(base_path,sprintf('analysis/SASTT/%s/SH/%s/optimization_output/', sample_name, params.add_name));
if ~exist(params.optimization_output_path, 'file')
    mkdir(params.optimization_output_path);
end
params.figures = fullfile(base_path,sprintf('analysis/SASTT/%s/SH/%s/figures/', sample_name, params.add_name));
if ~exist(params.figures, 'file')
    mkdir(params.figures);
end
params.results = fullfile(base_path,sprintf('analysis/SASTT/%s/SH/%s/results/', sample_name, params.add_name));
if ~exist(params.results, 'file')
    mkdir(params.results);
end


% load data
time_load = tic;
filename = fullfile(base_path,sprintf('analysis/SASTT/%s/projection_data/SASTT_%s_aligned_ASTRA.mat', ...
    sample_name, sample_name));
fprintf('Step 2.1 Load the data\n')
fprintf('Loading %s\n',filename)
load(filename);

% timing
time_load = toc(time_load);
filename = fullfile(params.optimization_output_path, sprintf('time_load_%s_%s.mat', sample_name, params.add_name));
save(filename, 'time_load')

%% Step 2.2 optimization of coefficients over the symmetric intensity: only a0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parameters for the optimization
params.find_orientation = 0;        % Optimize over the SH angles: theta and phi (true or false)
params.find_coefficients = 1;       % Optimize over SH coefficients: a (true or false)

params.regularization = 0;          % Sieves regularization on the coefficients. (true or false)
params.regularization_angle = 0;    % Regularization of the angles.  (true or false)

%parameters for the sieves regularization (blurring the gradient)
%for the coefficients only
params.slice = 0; %in case the optimization is done on a slice (so it is using a 2D kernel)
kernel3D = window3(5,5,5,@hamming);
params.kernel = kernel3D./sum(kernel3D(:)); % for normalization (sum equals 1)

params.itmax = 20;                  % maximum number of iterations: about 20
params.skip_projections = 1;               % = 1, for not skipping projections

params.coeff_soft_limits_low = -Inf;
params.coeff_soft_limits_high = Inf;
params.soft_limit_weight_coeff = 1e5;               % Penalty value of the soft limit

params.display = 1;
params.dispinter = 1;
params.plot = 1;

%parameters for the projection at arbitrary angles
params.volume_upsampling = 1;  % Upsamples the volume by a factor of 2 to reduce artifacts
params.method = 'bilinear';    % 'nearest' or 'bilinear' method for volume allocation
params.filter_2D = 1;          % Strength of filter applied to image

params.avoid_wrapping=0;            % avoid wrapping over 2Pi
params.a1_const = 0;                % first SH coefficient is fixed (not optimizied) if true

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time_sym = tic;


% %%% 2D plot characteristics (the data) %%%
% Half of the Angular segments in integration
numsegments = length(projection(1).integ.phi_det)/2;
params.numsegments = numsegments;
% read the phi_det from integ
% consider only the first half because of the symmetry assumption for the
% segments
params.phi_det = deg2rad(projection(1).integ.phi_det(1:numsegments)); %in radians
%%%%%%%%%%%%%%%%%%%%

% define the size of the tomogram (not the same as the tomogram, as some projection might change the size)
dim_x = 0;
dim_y = 0;
for i = 1:length(projection)
    dim_x1 = size(projection(i).data, 2);
    dim_y1 = size(projection(i).data, 1);
    if dim_x1 > dim_x
        dim_x = dim_x1;
    end
    if dim_y1 > dim_y
        dim_y = dim_y1;
    end
end
lines = (dim_x);%+6
points_per_line = (dim_y);%+6

% number of voxels in the tomogram
numOfvoxels = lines * lines * points_per_line;
%save in params
params.lines = lines;
params.points_per_line = points_per_line;
params.numOfvoxels = numOfvoxels;

mask3D = ones(points_per_line, lines, lines);
params.mask3D = mask3D;
% parameters for optimization of 1 coefficient for symmetric intensity
% Spherical harmonic parameters for optimizing only the coefficient
% this is the same as fitting a voxel with a certain intensity - no angle
% available
l = 0;  % Polar order: degree
m = 0;  % Azimuthal order
a = 0.0001;   % initial guess for Coefficient
%store in params
params.l = l;                           % polar degree of spherical harmonics
params.m = m;                       % azimuthal order of spherical harmonics
%%% Initial values for the SH angles
theta_init = pi/4; %45 degrees:  Polar orientation of structure
phi_init=pi/4; %45 degrees: Azimuthal orientation of structure
% calculate the initial values
theta_0_init = ones(1,numOfvoxels)*theta_init; % Polar orientation of structure
phi_0_init = ones(1,numOfvoxels)*phi_init;     % Azimuthal orientation of structure
a_init = ones(1, numOfvoxels).*a(1);

%optimize
fprintf('Step 2.2 optimization of coefficients over the symmetric intensity: only a0\n')
[params, theta_out1, phi_out1, a_out1] = ...
    optimization.optimize_SH(projection, params, theta_0_init, phi_0_init, a_init);

% saving the results
% load the result from cgmin1
load(sprintf('%s/current_opt_symsaxs.mat', params.optimization_output_path));
%save the results
filename = sprintf('%s/result_%s_q%d-%d_symSAXS_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
save(filename,'a_out1','theta_out1','phi_out1','e', 'params', '-v6');
fprintf('Saving results in %s\n',filename)
print(gcf, sprintf('%s/optimization_sym_int_%s.jpg', params.figures, params.add_name),'-djpeg','-r300');


time_sym = toc(time_sym);
filename = fullfile(params.optimization_output_path, sprintf('time_sym_%s_%s.mat', sample_name, params.add_name));
save(filename, 'time_sym')

%% Step 2.3: Define a 3D mask (voxels that aren't going to be optimized)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
make_3Dmask = 1; % 1 to create a 3D mask, 0 to see the symmetric intensity tomogram
mask.cut_off = []; % = [] for automatic detection of the mean value using Otsu threshold
mask.diam_cylinder = []; % Diameter of a cylinder outside, = [] does not create the mask
mask.gauss_filter = 1; % Apply a filter to the data to reduce noise
mask.openning = 1;     % Apply an kind of openning operation on the mask, this makes it larger and is important to not have too tight constraint.
coloraxis = 'auto'; % apply same color scale to the three projections xy, yz and xz fron view3
viewpar.interactive = 1;  % = true if you want to explore the volume
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Step 2.3: Define a 3D mask (voxels that aren''t going to be optimized)\n')

% load the result from symmetric intensity
filename_sym_int = sprintf('%sresult_%s_q%d-%d_symSAXS_%s.mat', params.optimization_output_path, sample_name, ...
    projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
fprintf('Loading %s',filename_sym_int)
load(filename_sym_int);

time_mask = tic;
% needed for plotting
a_out1 = a_out1(1:params.numOfvoxels);
a_tomo_sym = reshape(a_out1, params.points_per_line, params.lines, params.lines);
    
if make_3Dmask
   [params.mask3D, cut_off] = create_mask3D(a_tomo_sym, mask);
else
    params.mask3D = ones(size(a_tomo_sym));
    cut_off = 0;
end

%save the updated 3Dmask in symmetric intensity
fprintf('Saving the updated 3D mask in %s', filename_sym_int)
save(filename_sym_int,'a_out1','theta_out1','phi_out1','e', 'params', '-v6');

% saving the timing results
time_mask = toc(time_mask);
filename = fullfile(params.optimization_output_path,sprintf('time_mask_%s_%s.mat', sample_name, params.add_name));
save(filename, 'time_mask')

% plot the masked symmetriuc intensity reconstruction
view3(params.mask3D.*a_tomo_sym + max(a_tomo_sym(:))*(1-params.mask3D), coloraxis, viewpar)
colormap(plotting.franzmap);

%% Step 2.4: optimization of SH angles (phi and theta)
time_angle = tic;
filetoload = sprintf('%sresult_%s_q%d-%d_symSAXS_%s.mat',  params.optimization_output_path, sample_name, ...
    projection(1).par.r_sum{1}(1), projection(1).par.r_sum{1}(end), params.add_name);
fprintf('Step 2.4: optimization of SH angles (phi and theta)\n')
fprintf('Loading %s\n',filetoload)
sym_int = importdata(filetoload);
params = sym_int.params;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
params.find_orientation = 1;                % Optimize over the main orientation of the structure (true or false)
params.find_coefficients = 0;               % Optimize over coefficients (true or false)

params.regularization = 0;                  % Sieves regularization on the coefficients. (true or false)
params.regularization_angle = 0;            % Regularization of the angles. (true or false)
params.regularization_angle_coeff = 0;      % mu for regularization of angle, needs to be found with L-curve

%%%define ratio of coefficient (fixed in this step) for bone: 1, -3, 6,
params.a1_init = 1;
params.a2_init = -3;   % initial guess for a2 relative to a1 (symmetric intensity). such that a2 = a1/params.a2_init
params.a3_init = 6;    % similar as above a3 with respect to a1
params.a_out1 = sym_int.a_out1;
params.itmax = 30;                          % maximum number of iterations: about 50
params.skip_projections = 1;                       % = 1, for not skipping projections

params.coeff_soft_limits_low = -Inf;
params.coeff_soft_limits_high = Inf;
params.soft_limit_weight_coeff = 1e5;                         % Penalty applied to the soft limits

params.display = 1;
params.dispinter = 1;
params.plot = 1;

%parameters for the projection at arbitary angles
params.volume_upsampling = 1;  % Upsamples the volume by a factor of 2 to reduce artifacts
params.method = 'bilinear';    % 'nearest' or 'bilinear' method for volume allocation
params.filter_2D = 1;          % Strength of filter applied to image

params.avoid_wrapping = 1;                  % Avoid wrapping of the angle (to keep angle between 0 and 2pi) (true or false)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do optimization
close all

% parameters for angles only
% values for initial guess of coeffiecient, relative to symmetric intensity
% coefficients of the SH
% Vector with spherical harmonic parameters
l = [0  2  4 ];  % Polar order
m = [0   0  0];  % Azimuthal order
a = [0.001  -0.0001 0.0001];   % Coefficients (not used)
params.l = l;                % polar order of spherical harmonics
params.m = m;                % azimuthal order of spherical harmonics
a_out1 = params.a_out1;

lines = params.lines;
points_per_line = params.points_per_line;
numOfvoxels = params.numOfvoxels;

%%intital values of angles
theta_init = pi/4;
phi_init=pi/4;
theta_0_init = ones(1,numOfvoxels)*theta_init;           % Polar orientation of structure
phi_0_init   = ones(1,numOfvoxels)*phi_init;           % Azimuthal orientation of structure
a_tomo=reshape(a_out1,points_per_line, lines, lines);
a_init=zeros(points_per_line, lines, lines,length(l));
a_init(:,:,:,1)=a_tomo./params.a1_init;
a_init(:,:,:,2)=a_tomo./params.a2_init;
a_init(:,:,:,3)=a_tomo./params.a3_init;
a_init = reshape(a_init,1,[]);
params.a = a_init;

%optimize
[params, theta_out1, phi_out1, a_out1] = ...
    optimization.optimize_SH(projection, params, theta_0_init, phi_0_init, a_init);

if params.regularization_angle
    load(sprintf('%s/current_opt_saxs_angle.mat', params.optimization_output_path));
    filename = sprintf('%s/result_%s_q%d-%d_angles_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
    save(filename,'a_out1','theta_out1','phi_out1','e','reg_error', 'params', '-v6');
else
    load(sprintf('%s/current_opt_saxs_angle.mat', params.optimization_output_path));
    filename = sprintf('%s/result_%s_q%d-%d_angles_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
    save(filename,'a_out1','theta_out1','phi_out1','e', 'params', '-v6');
end
print(gcf, sprintf('%s/optimization_angles_%s.jpg', params.figures, params.add_name),'-djpeg','-r300');
fprintf('Saved result to %s\n',filename)

% timing
time_angle = toc(time_angle);
filename = fullfile(params.optimization_output_path,sprintf('time_angle_%s_%s.mat', sample_name, params.add_name));
save(filename, 'time_angle')
%% Step 2.5: optimization of SH coefficients: non symmetric
% optimize the other coefficients a2, a4 and a6, keeping a1 constant
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
params.find_orientation = 0;                    % Optimize over the main orientation of the structure (true or false)
params.find_coefficients = 1;                   % Optimize over coefficients (true or false)
params.regularization = 1;                      % apply Sieves regularization on the coefficient (true or false)
params.regularization_angle = 0;                % Regularization of the angles. (true or false)

%parameters for the sieves regularization (blurring the gradient)
params.slice = 0;
kernel3D = window3(5,5,5,@hamming);             % strength of Sieves regularization
params.kernel = kernel3D./sum(kernel3D(:));     % for normalization (sum equals 1)

params.itmax = 20;                              % maximum number of iterations: about 20
params.skip_projections = 1;                           % = 1, for not skipping projections

params.coeff_soft_limits_low = -Inf;
params.coeff_soft_limits_high = Inf;
params.soft_limit_weight_coeff = 1e5;

params.display = 1;
params.dispinter = 1;
params.plot = 1;

%parameters for the projection at arbitary angles
params.volume_upsampling = 1;  % Upsamples the volume by a factor of 2 to reduce artifacts
params.method = 'bilinear';    % 'nearest' or 'bilinear' method for volume allocation
params.filter_2D = 1;          % Strength of filter applied to image

params.avoid_wrapping = 0;                      % avoid wrapping over 2Pi of the angle
params.a1_const = 1;                            % keep the first coeffiecient a1 constant (true or false)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do optimization
time_coeff = tic;
close all

fprintf('Step 2.5: optimization of SH coefficients: non symmetric\n')

%%%%%%%%%% Initial conditions (for each voxel in the tomogram)
filetoload = sprintf('%s/result_%s_q%d-%d_symSAXS_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1), ...
    projection(1).par.r_sum{1}(end), params.add_name);
fprintf('Loading symmetric intensity results from %s\n',filetoload)
sym_opt = importdata(filetoload);

% take from the symmetric intensity
if params.a1_const
    params.a1= sym_opt.a_out1; %gives first coeff as parameter because it is kept constant
end


% parameters for optimization of coefficients with constant a1 fixed from initial optimization
% loas parameters from the angle
parameterfileload = sprintf('%s/result_%s_q%d-%d_angles_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1), ...
    projection(1).par.r_sum{1}(end), params.add_name);
fprintf('Loading parameters from %s\n',parameterfileload)
angle_opt = importdata(parameterfileload);


params.theta_0_init = reshape(angle_opt.theta_out1,1,[]);
params.phi_0_init   = reshape(angle_opt.phi_out1,1,[]);
params.phi_det = angle_opt.params.phi_det;
%%%%%%%%%%%%

lines = params.lines;
points_per_line = params.points_per_line;
numOfvoxels = params.numOfvoxels;

% Vector with spherical harmonic parameters
l = [0  2 4 6];  % Polar order
m = [0  0 0 0];  % Azimuthal order
a = [0.001  -0.0001 0.001 -0.0001];   % Coefficients
params.l = l;                % polar order of spherical harmonics
params.m = m;                % azimuthal order of spherical harmonics

% only the second to 4th coefficient are optimization inputs, the first one
% is given above as a constant parameter (params.a1)
if params.a1_const
    a_init=zeros(points_per_line, lines, lines,length(l)-1);
    a_init(:,:,:,1)=ones(points_per_line, lines, lines).*a(2);
    a_init(:,:,:,2)=ones(points_per_line, lines, lines).*a(3);
    a_init(:,:,:,3)=ones(points_per_line, lines, lines).*a(4);
    a_init = reshape(a_init,1,[]);
else
    a_init=zeros(points_per_line, lines, lines,length(l)-1);
    a_init(:,:,:,1)=ones(points_per_line, lines, lines).*a(1);
    a_init(:,:,:,2)=ones(points_per_line, lines, lines).*a(2);
    a_init(:,:,:,3)=ones(points_per_line, lines, lines).*a(3);
    a_init(:,:,:,4)=ones(points_per_line, lines, lines).*a(4);
    a_init = reshape(a_init,1,[]);
end
theta_0_init = params.theta_0_init;
phi_0_init = params.phi_0_init;

%optimize
[params, theta_out1, phi_out1, a_out1] = ...
    optimization.optimize_SH(projection, params, theta_0_init, phi_0_init, a_init);

% save the results
if params.regularization_angle
    load(sprintf('%s/current_opt_saxs.mat', params.optimization_output_path));
    filename = sprintf('%s/result_%s_q%d-%d_coef_a1const_%s.mat', params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
    save(filename,'a_out1','theta_out1','phi_out1','e','reg_error', 'params', '-v6');
else
    load(sprintf('%s/current_opt_saxs.mat', params.optimization_output_path));
    filename = sprintf('%s/result_%s_q%d-%d_coef_a1const_%s.mat',params.optimization_output_path, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
    save(filename,'a_out1','theta_out1','phi_out1','e', 'params', '-v6');
end
print(gcf, sprintf('%s/optimization_coef_%s.jpg', params.figures, params.add_name),'-djpeg','-r300');

% timing
time_coeff = toc(time_coeff);
filename = fullfile(params.optimization_output_path,sprintf('time_coeff_%s_%s.mat', sample_name, params.add_name));
fprintf('Saving result in %s\n',filename)
save(filename, 'time_coeff')

%% Step 2.6: find optimal regularization parameters for final step
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
it_max = 10;                           % 3 to 10 iterations seem to work well
mu = logspace(-5, 10, 15);           % This values should be standard: larger range if there is no crossing point
params.skip_projections = 10;                 % = 1, for not skipping projections. = 5-10 works well
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time_reg = tic;
close all
[errormetric, penalty_error, regularization_coeff] = find_regularization_mu(projection, mu, it_max, params);

% timing
time_reg = toc(time_reg);
filename = fullfile(params.optimization_output_path,sprintf('time_reg_%s_%s.mat', sample_name, params.add_name));
save(filename, 'time_reg')
%% Step 2.7: final optimization: combine all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EDIT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine an approximation of the regularization coefficient
filename = sprintf('%sresult_%s_q%d-%d_coef_a1const_%s.mat', params.optimization_output_path, sample_name, ...
        projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
load(filename)
a_tomo = reshape(a_out1(1:params.numOfvoxels), params.points_per_line, params.lines, params.lines);
t = sum(((a_tomo.*spherical_harm(0,0,0,0)).^2),4); %plot the first coefficent
tt = (sum(t, 3).*projection(1).window_mask); %axis tight equal xy
mu_guess = mean(mean(tt))
% optimize all coefficients (a0, a2, a4 and a6) and angles (theta and phi)
% whihout any constriction
params.find_orientation = 1;      % Optimize over the main orientation of the structure
params.find_coefficients = 1;     % Optimize over coefficients

params.regularization = 0;        % Sieves regularization on coeff
params.regularization_angle = 1; %regularization of angle
params.regularization_angle_coeff = mu_guess;  %find the appropriate mu with L-curve from step 2.6

%parameters for the sieves regularization (blurring the gradient)
kernel3D=window3(5,5,5,@hamming);
params.kernel=kernel3D./sum(kernel3D(:)); % for normalization (sum equals 1)

params.itmax = 50;                          % maximum number of iterations: about 50-100
params.skip_projections = 1;                % = 1, for not skipping projections

params.coeff_soft_limits_low = -Inf;
params.coeff_soft_limits_high = Inf;
params.soft_limit_weight_coeff = 1e5;

params.display = 1; %display intermediate results from optimization
params.dispinter = 1; % number of iterations step results are displayed
params.plot = 1; %plot intermediate results

%parameters for the projection at arbitary angles
params.volume_upsampling = 1;  % Upsamples the volume by a factor of 2 to reduce artifacts
params.method = 'bilinear';    % 'nearest' or 'bilinear' method for volume allocation
params.filter_2D = 1;          % Strength of filter applied to image

params.avoid_wrapping = 1;     % avoid wrapping over 2Pi of the angle
params.a1_const = 0;                 %keeping first coeff constnat (true false)

% provide the file used as initial guess
init_guess = load(sprintf('%sresult_%s_q%d-%d_coef_a1const_%s.mat', params.optimization_output_path, sample_name, ...
        projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% do optimizaton
% provide initial guess
params.theta_init = init_guess.theta_out1;
params.phi_init = init_guess.phi_out1;
params.a_init = init_guess.a_out1;

time_all = tic;
close all

% parameters for optimization of angles with the 4 coefficients
%%% Vector with spherical harmonic parameters
lines = params.lines;
points_per_line = params.points_per_line;
numOfvoxels = params.numOfvoxels;

l = [0  2 4 6];  % Polar order
m = [0  0 0 0];  % Azimuthal order
% a = [0.001  -0.0001 0.00001 -0.000001];   % Coefficients - not used
params.l = l;                % polar order of spherical harmonics
params.m = m;                % azimuthal order of spherical harmonics

%%% Initial conditions (for each voxel in the tomogram)
theta_0_init = params.theta_init;
phi_0_init   = params.phi_init;
a_init = zeros(points_per_line, lines, lines,numel(l));
if numel(a_init)>(numel(params.a_init))
    a_init(1:numel(params.a_init))=params.a_init(:);
else
    a_init(:)=params.a_init(1:numel(a_init));
end
a_init = reshape(a_init,1,[]);


% needed when IRTT results are given as initial guess
if ~isfield(params, 'phi_det')
    % %%% 2D plot characteristics (the data) %%%
    % Half of the Angular segments in integration
    numsegments = length(projection(1).integ.phi_det)/2;
    params.numsegments = numsegments;
    % read the phi_det from integ
    % consider only the first half because of the symmetry assumption for the
    % segments
    params.phi_det = deg2rad(projection(1).integ.phi_det(1:numsegments)); %in radians
    %%%%%%%%%%%%%%%%%%%%
end

%optimize
[params, theta_out1, phi_out1, a_out1] = ...
    optimization.optimize_SH(projection, params, theta_0_init, phi_0_init, a_init);

% save results
% load the result from cgmin1
load(sprintf('%scurrent_opt_saxs_all.mat', params.optimization_output_path));
filename = sprintf('%s/result_%s_q%d-%d_all_again_%s.mat', params.results, params.sample_name, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name);
save(filename,'a_out1','theta_out1','phi_out1','e', 'params', '-v6');
print(gcf, sprintf('%s/optimization_all_q%d-%d_%s.jpg', params.figures, projection(1).par.r_sum{1}(1),  projection(1).par.r_sum{1}(end), params.add_name),'-djpeg','-r300');

% timing
time_all = toc(time_all);
filename = fullfile(params.optimization_output_path,sprintf('time_all_%s_%s.mat', sample_name, params.add_name));
save(filename, 'time_all')